/*
 * Layout Manager
 * 
 * V0.0.1
 */

/* global utils, WebOSManager, X2J, Logger, application, LibraryManager, ScheduleManager, CacheManager */
(function(){
	"use strict";

	/**
	 * @classdesc This module is responsible to handle layouts.
	 * @type LayoutManager
	 * @version 1.8.1
	 * @since 1.8.1
	 */
	//jshint -W057
	window.LayoutManager = new function _LayoutManager() {
		/**
		 * The name of the html element that hosts the layouts
		 */
		this.factory = function(type, idx, e, mediacallback, layoutid, scheduleid, scp){
			if (this.hasOwnProperty(type)){
				return new this[type](idx, e, mediacallback, layoutid, scheduleid, scp);
			}else{
				console.warn("Unsupported type : " + type);
				return null;
			}        
		};

		/** The HTML element to display content */
		var targetElementName = null;
		/** The last Layout that has been displayed */
		var lastShowPage = null;
		/** The array with layouts in queue */
		var v_RunningLayouts = [];
		var v_displayOverLayLayout = [];
		/** The array delivered from Scheduler with the layouts to be parsed */
		var v_NewLayouts = [];
		/** The currently displayed layout index */
		var v_CurrentLayoutIdx = 0;
		/** */
		var v_parsedLayouts = [];
		/** The regions of the current layout. */
		var v_CurrentLayoutRegions = {};

		/** The num of active overlay layouts. */
		var v_activeOverlay = 0;

		/** Flag if schedule is in pause state in order to so non scheduled screens like settings */
		var v_ScheduleIsPaused = false;

		/** Should the layout start playing */
		var shouldStartPlaying = false;

		/**
		 * Reference to timers.
		 */
		var LayoutElements = [];

		/* */
		var layoutStartedTime = null;

		/**
		 * Check if manager is started!
		 */
		var v_isStarted = false;

		/**
		 * Flag to prevent displaying external input
		 */
		this.shouldDisplayExternalInput = true;
	    /**
	     * Array with Layout ids that are invalid.
	     * Possible reasons are:
	     * Region is empty.
	     */
	    this.invalidLayoutIds = [];

		/* +------------------------+ Private functions +------------------------+ */

		/* Start playing media */
		var displayMedia = function(mediaSequence, nextMediaIndex, regionIndex, layout, overlayIndex){
			//console.log('Display media',mediaSequence.media[nextMediaIndex], 'on region', regionIndex);
			//nextMediaIndex++;
			//if (defined(LayoutElements[layout.id][mediaSequence.regionID])) { LayoutElements[layout.id][mediaSequence.regionID].stopMediaTimeout(); }
			var layoutRegionsIndex;
			if (layout.isoverlay) {
				layoutRegionsIndex = "overlay" + overlayIndex;
			} else {
				layoutRegionsIndex = layout.id;
			}
			if (nextMediaIndex === 0) {
				layoutStartedTime = utils.getISODate();
			}
			var e = document.getElementById(mediaSequence.regionID);
	        if (e === null){
	            return;
	        }

			var shouldnotloop = false;
			var shouldnotloopByContainer = false;
			if (e){
				shouldnotloopByContainer = (e.childElementCount !== 0);
			}
			if ((layout.regions[regionIndex].loop === 0) && (mediaSequence.media.length === 1) && shouldnotloopByContainer) {
				shouldnotloop = true;
			}
			if (nextMediaIndex >= mediaSequence.media.length) {
				nextMediaIndex = 0;
				mediaSequence.sequenceExpired = true;
				v_CurrentLayoutRegions[layoutRegionsIndex][regionIndex].regionExpired = true;
				/* call Region Expired */
				if (!layout.isoverlay && allRegionsExpired(layoutRegionsIndex))
				{
					if (layout.id !== 'splash'){
						var ltodt = utils.getISODate();
						var dur = (((new Date())- new Date(layoutStartedTime))/1000)|0;
						CacheManager.setLayoutDuration(layout.id, dur);
						if (layout.enableStat !== "0") {
						  Logger.logStat({fromdt: layoutStartedTime, todt: ltodt, scheduleid: layout.scheduleid, layoutid: layout.id});
						}

					}
					//console.debug("All Regions Expired");
					if (layout.isoverlay) {
						displayLayouts(layout.isoverlay, overlayIndex);
					} else {
						displayLayouts();
					}
					return;
				}
			}
			//console.debug("Element "+ mediaSequence.media[nextMediaIndex].type +" should not loop", [(layout.regions[regionIndex].loop === 0),(mediaSequence.media.length === 1), (e.childElementCount !== 0)], (layout.regions[regionIndex].loop === 0) && (mediaSequence.media.length === 1) && (e.childElementCount !== 0));

			var mediacallback = function() {
				displayMedia(mediaSequence,  nextMediaIndex+1, regionIndex, layout, overlayIndex);
			};
			var nativetohtml = ["embedded", "ticker", "text", "webpage", "datasetview"];
			var scp;
			if (defined(overlayIndex)) {
				scp = "overlay";
			} else {
				scp = "main";
			}
//			var skipthis = false;
//			if (mediaSequence.media[nextMediaIndex].hasOwnProperty('shouldnotloop')) {
//			if (mediaSequence.media[nextMediaIndex].hasOwnProperty('shouldnotloop') === true) {
//			skipthis = true;                
//			}
//			}
			if (!(shouldnotloop)) {
				var mediaElement;
				if (mediaSequence.media[nextMediaIndex].render === "native") {
					if (nativetohtml.indexOf(mediaSequence.media[nextMediaIndex].type) > -1) {
						mediaElement = LayoutManager.factory("html", mediaSequence.media[nextMediaIndex], e, mediacallback, layout.id, layout.scheduleid, scp);
					} else {
						mediaElement = LayoutManager.factory(mediaSequence.media[nextMediaIndex].type, mediaSequence.media[nextMediaIndex], e, mediacallback, layout.id, layout.scheduleid, scp);
					}
					if (mediaElement === null) { mediacallback(); }
				} else if (mediaSequence.media[nextMediaIndex].render === "html") {
					mediaElement = LayoutManager.factory("html", mediaSequence.media[nextMediaIndex], e, mediacallback, layout.id, layout.scheduleid, scp);
					if (mediaElement === null) { mediacallback(); }
				} else {
					mediacallback();
				}
				if (layout.isoverlay) {
					LayoutElements["overlay"+overlayIndex][mediaSequence.regionID] = mediaElement;
				} else {
					LayoutElements[layout.id][mediaSequence.regionID] = mediaElement;
				}
			}
//			if (shouldnotloop) {
//			mediaSequence.media[nextMediaIndex].shouldnotloop = true;
//			}
		};

		/* Media ended */
		/* Region expired */
		var allRegionsExpired = function(layoutRegionsIndex){
			//console.debug("Checking expired regions");
			var layoutsExpired = true; /* Reverse logic */
			for (var i = 0; i < v_CurrentLayoutRegions[layoutRegionsIndex].length; i++) {
				//console.debug(i + " | Region " + v_CurrentLayoutRegions[layoutRegionsIndex][i].id + " expired " + v_CurrentLayoutRegions[layoutRegionsIndex][i].regionExpired);
				if (!v_CurrentLayoutRegions[layoutRegionsIndex][i].hasOwnProperty("regionExpired"))
				{
					layoutsExpired = false;
					break;
				}else if (v_CurrentLayoutRegions[layoutRegionsIndex][i].regionExpired === false) {
					layoutsExpired = false;
					break;
				}
			}

			return layoutsExpired;
		};


		var displayLayouts = function(isoverlay, overlayIndex) {
			if (isoverlay === true) {
				displayLayout(null, true, overlayIndex);
			} else {
				/* if we show something like settings screen, don't change layouts */
				if (v_ScheduleIsPaused) { return; }
				v_CurrentLayoutIdx++;
				lastShowPage = v_CurrentLayoutIdx;
				if (v_CurrentLayoutIdx >= v_RunningLayouts.length) { v_CurrentLayoutIdx = 0; }
				displayLayout(v_CurrentLayoutIdx);
				var nextLayoutIdx=v_CurrentLayoutIdx+1;
				if (nextLayoutIdx >= v_RunningLayouts.length) { nextLayoutIdx = 0; }
	            application.statusPage.currentLayout(v_RunningLayouts[v_CurrentLayoutIdx].id);
				application.statusPage.nextLayout(v_RunningLayouts[nextLayoutIdx].id);
			}
		};

		var displayLayout = function(layoutIndex, isoverlay, overlayIndex){
			var v_Layout;
			var e;
			var overlaycontainer;
			var overlayinnercontainer;
			var overlayoutercontainer;
			var scaleroptions;
			var layoutRegionsIndex;
			var layoutElementsToRemove = [];
			if (isoverlay === true) {
				overlayoutercontainer = document.createElement("div");
				overlaycontainer = document.getElementById('overlayContainer');
				overlayinnercontainer = document.createElement("div");
				e = document.createElement("div");
				overlayinnercontainer.appendChild(e);
				overlayoutercontainer.appendChild(overlayinnercontainer);
				overlaycontainer.appendChild(overlayoutercontainer);
				v_Layout = v_displayOverLayLayout[overlayIndex];
				utils.empty("#overlay-"+v_Layout.id+"-outer");
				utils.remove("#overlay-"+v_Layout.id+"-outer");
				v_Layout.isoverlay = isoverlay;
				e.setAttribute('id',"overlay-"+v_Layout.id);
				overlayinnercontainer.setAttribute('id',"overlay-"+v_Layout.id+"-innercontainer-"+overlayIndex);

				overlayinnercontainer.setAttribute('style',"left: 0; top: 0; position: absolute; width: " + document.body.getBoundingClientRect().width + "px; height: " + document.body.getBoundingClientRect().height + "px;");

				if(DeviceManager.Device.getOrientation() == DeviceManager.Device.SCREENORIENTATION_PORTRAIT|| DeviceManager.Device.getOrientation() == DeviceManager.Device.SCREENORIENTATION_REVERSE_PORTRAIT){
					if(v_Layout.width > v_Layout.height) {
						overlayinnercontainer.style.top = screen.height / 2 -(v_Layout.height*(screen.width/v_Layout.width))/2 + 'px';
					}
				}else{
					if(v_Layout.width < v_Layout.height){
						overlayinnercontainer.style.left = screen.width / 2 -(v_Layout.width*(screen.height/v_Layout.height))/2 + 'px';
					}
				}
				overlayoutercontainer.setAttribute('id',"overlay-"+v_Layout.id+"-outer");
				layoutRegionsIndex = "overlay" + overlayIndex;
				if (!defined(LayoutElements[layoutRegionsIndex])) {
					LayoutElements[layoutRegionsIndex] = [];
				}            
			}
			else {
				e = document.getElementById(targetElementName);
				e.innerHTML = null;
//				while (e.firstChild) {
//				e.removeChild(e.firstChild);
//				}
				if (v_RunningLayouts.length === 0) {
					v_RunningLayouts.push(LayoutParser.getDefault());
				}
				v_Layout = v_RunningLayouts[layoutIndex];
				clearLayoutTimeouts("main", v_Layout.id);
				layoutRegionsIndex = v_Layout.id;
				LayoutElements.forEach(function(layout) {
					layout.forEach(function(element) {
	                  if (defined(element)){
						element.stopMediaTimeout();
	                  }
					});
				});
				
				var layoutElemKeys = Object.keys(LayoutElements);
				for(var i = 0; i < layoutElemKeys.length; i++){
					if (!layoutElemKeys[i].startsWith("overlay")){
						delete LayoutElements[layoutElemKeys[i]];
					}
				}
				LayoutElements[v_Layout.id] = [];
			}
			//e.style.width = v_Layout.width+'px';
			//e.style.height = v_Layout.height+'px';

			e.style.width = screen.width+'px';
			e.style.height = screen.height+'px';
			
			if (!defined(isoverlay)) { 
				e.style.backgroundColor = v_Layout.bgcolor;
			}
			
			scaleroptions = {
					"originalWidth": v_Layout.width,
					"originalHeight": v_Layout.height,
					"previewWidth": utils.getElementWidth(e),
					"previewHeight": utils.getElementHeight(e),
					"scaleOverride": 0,
					"widgetDesignWidth": v_Layout.width,
					"widgetDesignHeight": v_Layout.height,
					"widgetDesignPadding": 0,
					"itemsPerPage": 0
			};
			var el;
			if (isoverlay === true) {
				//jQueryWebOs("#overlay-"+v_Layout.id+"-innercontainer-"+v_activeOverlay).xiboLayoutScaler(scaleroptions);
				utils.scaler("#overlay-"+v_Layout.id+"-innercontainer-"+v_activeOverlay,scaleroptions);
				el = utils.elementSelector("#overlay-"+v_Layout.id+"-innercontainer-"+v_activeOverlay+" > div");
				el.style.margin = "auto";
				el.style.top = "auto";
				el.style.left = "auto";
				el.style.bottom = "auto";
				el.style.right = "auto";
			} else {

				//jQueryWebOs("#mainContentContainer").xiboLayoutScaler(scaleroptions);
				utils.scaler("#viewContainer",scaleroptions);
				if (v_Layout.id !== 'splash'){
					viewContainer.style.top =0;
					viewContainer.style.left =0;
					if(DeviceManager.Device.getOrientation() == DeviceManager.Device.SCREENORIENTATION_PORTRAIT|| DeviceManager.Device.getOrientation() == DeviceManager.Device.SCREENORIENTATION_REVERSE_PORTRAIT){
						if(v_Layout.width > v_Layout.height) {
							viewContainer.style.top = screen.height / 2 -(v_Layout.height*(screen.width/v_Layout.width))/2 + 'px';
						}
					}else{
						if(v_Layout.width < v_Layout.height){
							viewContainer.style.left = screen.width / 2 -(v_Layout.width*(screen.height/v_Layout.height))/2 + 'px';
						}
					}
				}


//				el = utils.elementSelector("#mainContentContainer > div");
//				el.style.margin = "auto";
//				el.style.top = "auto";
//				el.style.left = "auto";
//				el.style.bottom = "auto";
//				el.style.right = "auto";
			}

			if (v_Layout.background !== "") {
//				e.style.backgroundImage = "url('"+v_Layout.background+"')";
//				e.style.backgroundSize = "cover";
				utils.setBackground(e,v_Layout.background);
			} else {
				//e.style.backgroundImage = "none";
				utils.removeBackground(e);
			}
			//e.style.position = "absolute";
			//e.style.zIndex = "1000";

			//v_CurrentLayoutRegions = {};
			v_CurrentLayoutRegions[layoutRegionsIndex] = [];

			for (var i = 0; i < v_Layout.regions.length; i++) {
				var nodeRegion = document.createElement("div");
				if (isoverlay === true) {
					nodeRegion.id = "overlay_" + v_activeOverlay + "_" + v_Layout.regions[i].id;
				} else {
					nodeRegion.id = v_Layout.regions[i].id;
				}

				v_Layout.regions[i].regionExpired = false;
				v_CurrentLayoutRegions[layoutRegionsIndex].push(v_Layout.regions[i]);
				nodeRegion.style.position = "absolute";
				//nodeRegion.style.background = "red";
				nodeRegion.style.width = v_Layout.regions[i].width;
				nodeRegion.style.height = v_Layout.regions[i].height;
				nodeRegion.style.top = v_Layout.regions[i].top;
				nodeRegion.style.left = v_Layout.regions[i].left;
				nodeRegion.style.display = 'flex';
				nodeRegion.style.justifyContent = 'center';
				nodeRegion.style.alignItems = 'center';
				if (defined(v_Layout.regions[i].zindex)){
					nodeRegion.style.zIndex = v_Layout.regions[i].zindex;
				}
				//console.debug("Number of children: " + nodeRegion.childElementCount);
				/* Prepare media sequence */
				var mediaSequense = {
						regionID : nodeRegion.id,
						currentMedia:0,
						sequenceExpired : false,
						media : []
				};
				for (var j = 0; j < v_Layout.regions[i].media.length; j++) {
					mediaSequense.media.push(v_Layout.regions[i].media[j]);
				}
				e.appendChild(nodeRegion);
				displayMedia(mediaSequense, 0, i, v_Layout, overlayIndex);
			}
			//console.debug("Layout regions", v_CurrentLayoutRegions);
		};


		/* +------------------------+ Public functions +------------------------+ */
		this.init = function (target) {
			targetElementName = target;
			v_RunningLayouts.push(LayoutParser.getDefault());
			/*LayoutManager.startPlaying();*/
			/*var options = {'layouts': v_NewLayouts};
        LayoutParser.parse(
            function() {
                v_parsedLayouts = this;
                v_parsedLayouts.forEach(
                    function(pushlayout) {
                        v_RunningLayouts.push(pushlayout);
                    }
                );
            },
            null,
            options
        );*/
		};


		/* Create a single audioContext to play all audio files! */
		this.audioContext = new AudioContext();

		this.hideExternalInputWidget = function(){
			LayoutManager.shouldDisplayExternalInput = false;
			var layoutElementKeys = Object.keys(LayoutElements);
			for(var i = 0; i < layoutElementKeys.length; i++){
				var widgetKeys = Object.keys(LayoutElements[layoutElementKeys[i]]);
				for(var j = 0; j < widgetKeys.length; j++){
					if(LayoutElements[layoutElementKeys[i]][widgetKeys[j]] instanceof LayoutManager.videoin){
							LayoutElements[layoutElementKeys[i]][widgetKeys[j]].hideMedia();
					}
				};
			}
		};
		
		this.showExternalInputWidget = function(){
			var isStatusPageVisible = document.getElementById("statusContainer").style.visibility === "visible";
			var isAppMenuBarVisible = document.getElementById("appMenu").className === "showMenu";
			var isLogContainerVisible = document.getElementById("logContainer").style.display === "block";
			if(isStatusPageVisible || isAppMenuBarVisible || isLogContainerVisible){
				return;
			}
			LayoutManager.shouldDisplayExternalInput = true;
			var layoutElementKeys = Object.keys(LayoutElements);
			for(var i = 0; i < layoutElementKeys.length; i++){
				var widgetKeys = Object.keys(LayoutElements[layoutElementKeys[i]]);
				for(var j = 0; j < widgetKeys.length; j++){
					if(LayoutElements[layoutElementKeys[i]][widgetKeys[j]] instanceof LayoutManager.videoin){
						LayoutElements[layoutElementKeys[i]][widgetKeys[j]].showMedia();
					}
				};
			}
		};

		this.startPlaying = function (isoverlay, overlayIndex) {
			if (!v_isStarted){
				v_isStarted = true;
				v_ScheduleIsPaused = false;
				v_CurrentLayoutIdx = -1;
				LayoutElements.forEach (
					function (item) {
						item.forEach (
							function (media) {
			                  if (defined(media)){
								media.stopMediaTimeout();
			                  }
							}      
						);
					}
				);
				displayLayouts();
			} else if (shouldStartPlaying) {
				removeSplash();

				shouldStartPlaying = false;
				LayoutElements.forEach (
					function (item) {
						item.forEach (
							function (media) {
							  if (defined(media)){
								media.stopMediaTimeout();
							  }
							}      
						);
					}
				);

				displayLayouts();


			}
			if (isoverlay) {
				displayLayouts(isoverlay, overlayIndex);
			}
		};

		this.stopPlaying = function () {
			clearLayoutTimeouts("main", null);
			clearLayoutTimeouts("overlay", null);
			utils.elementSelector("#viewContainer").style.backgroundColor = 'transparent';
			utils.elementSelector("#viewContainer").style.backgroundImage = 'none';
			utils.empty('#viewContainer');
			utils.empty('#overlayContainer');
	        lastShowPage = null;

		};


		/**
		 * Show the settings screen!
		 * @param {type} cbFunction Function to call when the user press the Connect to CMS button.
		 * @returns {undefined}
		 */
		this.showSettingsScreen = function (cbFunction) {
			LayoutManager.stopPlaying();
			v_isStarted = false;
			if (lastShowPage !== "cmsSettings.html"){
				utils.loadPagetoTarget("cmsSettings.html","connectCMSContainer", "documentLoad", cbFunction);
				lastShowPage = "cmsSettings.html";
				document.getElementById("connectCMSContainer").style.display = "block";
			}
		};
        
        this.hideSettingsScreen = function(){
		  document.getElementById('connectCMSContainer').style.display = "none";
		  document.getElementById("connectCMSContainer").innerHTML = "";
        };

		this.toast = function(cbSuccess, cbFailure, message){
			if (message === undefined){
				throw { 
					name:        "Parameter Error", 
					level:       "Show Stopper", 
					message:     "Message could not be undefined.", 
					htmlMessage: "Message could not be undefined.",
					toString:    function(){return this.name + ": " + this.message;} 
				}; 
			}
			if (true){
				var e = document.getElementById("toastMessage");
				e.innerHTML = message;
				setTimeout(function (){
					e.innerHTML = "";
				},5000);
				if (cbSuccess){
					cbSuccess();
				}
			}else{
				DeviceManager.Device.showMessage(cbSuccess,cbFailure,message);
			}
		};

		this.getCurrentLayoutId = function(){
			if (v_RunningLayouts.length === 0) {
				return null;
			}
	        if (defined(v_RunningLayouts[v_CurrentLayoutIdx])){
				if (v_RunningLayouts[v_CurrentLayoutIdx].hasOwnProperty("id")){
					return v_RunningLayouts[v_CurrentLayoutIdx].id;
				}else{
					return null;
				}
	        }else{
	            return null;
	        }
        };

		var removeSplash = function() {
			if (v_RunningLayouts.length > 0) {
				if (v_RunningLayouts[0].id === "splash") {
					v_RunningLayouts = [];
				}
			}
			if (defined(LayoutElements.splash)) {
				LayoutElements.splash.default_1.stopMediaTimeout();
				delete LayoutElements.splash;
				//LayoutElements.splice(LayoutElements.indexOf('splash'),1);
			}

		};

		/**
		 * Set the next layouts to play from Schedule Manager.
		 * @param {Array} layouts Array of layouts
		 * @see If the current layout is in the new layouts array then we should let it expire and then play the new layouts else start immediately.
		 **/

		this.setLayouts = function(layouts){
			if (parseOverlayLayout === null) {
				v_ScheduleIsPaused = false;
				//console.debug("Layouts:", layouts);
				var layoutsids = [];
	            //application.statusPage.scheduledLayouts_schedule.removeAll();
	            layouts.forEach(function(item) { 
	              layoutsids.push(item.layoutId); 
	              //application.statusPage.scheduledLayouts_schedule.push(item.layoutId);
	            } );
	            //application.statusPage.scheduledLayouts(layoutsids.join(", "));
				if (!v_RunningLayouts.hasOwnProperty(v_CurrentLayoutIdx)){
					console.log("Invalid layout index ("+v_CurrentLayoutIdx+").");
					v_CurrentLayoutIdx = 0;
					shouldStartPlaying = true;
				} else if (layoutsids.indexOf(v_RunningLayouts[v_CurrentLayoutIdx].id) === -1) {
					console.debug("Layout "+v_RunningLayouts[v_CurrentLayoutIdx].id+" not in array"); 
					v_CurrentLayoutIdx = -1;
					shouldStartPlaying = true;
				} else {
					console.debug("Layout "+v_RunningLayouts[v_CurrentLayoutIdx].id+" in array"); 
					shouldStartPlaying = false;
				}
				//console.debug('SHOULD START PLAYING: ' + shouldStartPlaying); 
				//v_RunningLayouts = [];
				//console.debug('Layouts', layouts);
				v_NewLayouts = layouts;
				var options = {'layouts': v_NewLayouts};
				//removeSplash();

				LayoutParser.parse(
						function() {
							v_parsedLayouts = this;
		                  	if (v_parsedLayouts.length===0){
								var schedData = ScheduleManager.getScheduleData();
								if (defined(schedData)){
									if (ScheduleManager.getScheduleData().hasOwnProperty('default')){
									  if(ScheduleManager.getScheduleData().default.isValid){
										LayoutParser.parse(
											function() {
												v_parsedLayouts = this;
												v_RunningLayouts = [];
												v_parsedLayouts.forEach(
													function(l) {
														v_RunningLayouts.push(l);
													}
												);
												//console.debug("Running layouts:",v_RunningLayouts);
												if (shouldStartPlaying) {
													LayoutManager.startPlaying();
												}
											},
											null,
											{'layouts': [{layoutId:ScheduleManager.getScheduleData().default.file}]}
										);
									  }else{
										  v_RunningLayouts = [];
										  v_RunningLayouts.push(LayoutParser.getDefault());
										  v_parsedLayouts.forEach(
											  function(pushlayout) {
												  v_RunningLayouts.push(pushlayout);
											  }
										  );
									  }
									}
								}
		                    }else{
		                      if (v_parsedLayouts.length>0){
		                        v_RunningLayouts = [];												
								v_parsedLayouts.forEach(
										function(pushlayout) {
											v_RunningLayouts.push(pushlayout);
										}
								);
		                      }
	                    	}			                  
							//console.debug("Running layouts:",v_RunningLayouts);
							if (shouldStartPlaying) {
								LayoutManager.startPlaying();
							}
						},
						null,
						options
				);
			}
		};

		/**
		 * Set the next overlay layouts to play from Schedule Manager.
		 * @param {Object} overlay Object with layoutID and scheduleID of overlay layout to show.
		 * @returns None
		 * @see 
		 * Multiple overlay layouts can be shown.<br>
		 * Overlay layout is a standard layout than can be scheduled to play on top of the currently playing layout.<br>
		 * The overlay layout is scaled to match the screen dimension as the normal layout does maintaining the aspect ratio.<br>
		 * If the background of the overlay layout is just a color then the background is transparent else if it has an image, the image is displayed.
		 **/
		var parseOverlayLayout = null;
		this.showOverlay = function(overlay){
			console.debug("Show Overlay layout "+overlay.layoutId);
			v_activeOverlay++;
			var parseOverLayLayout = [overlay];
			//v_displayOverLayLayout = overlay;
			var options = {'layouts': parseOverLayLayout};
			LayoutParser.parse(
					function() {
						v_parsedLayouts = this;
						v_parsedLayouts.forEach(
								function(pushlayout) {
									v_displayOverLayLayout[v_activeOverlay] = pushlayout;
									v_displayOverLayLayout[v_activeOverlay].overlayIndex = v_activeOverlay;
								}
						);
						LayoutManager.startPlaying(true, v_activeOverlay);
					},
					null,
					options
			);
		};

		/**
		 * Remove a previously shown layout.
		 * @param {String} overlay ID on overlay to remove.
		 * @returns None
		 */
		this.removeOverlay = function(overlay){
	      try {
			console.debug("Remove Overlay layout "+overlay.layoutId);
			var e = utils.elementSelector("#overlay-"+overlay.layoutId+"-outer");

			/* If strange things happen then this is something we have to implement! */

//			e.children[0].children[0].children.each(function() {
//			console.log(this);
//			for (var i = 0, max = this.children.length; i < max; i++) {
//			utils.off(this.childer[i]);
//			}
//			//jQueryWebOs(this).children("video").off();
//			});

//			jQueryWebOs("#overlay-"+overlay.layoutId+"-outer").children().first().children().first().children().each(function() {
//			jQueryWebOs(this).children("video").off();
//			});
			utils.empty(e.children[0]);
			utils.remove(e);
	      } catch (e) {
            console.error("Error occured while remove overlay layout", e);
          }
		};
		
		
	    
	    /** Orientation Changed 
	     * @version 1.8.1
	     * @since 1.8.1
	     */
	    this.orientationChanged = function(){
	      //Main content
	      if (v_RunningLayouts.length===0){
	        return;
	      }
	      var v_Layout= v_RunningLayouts[v_CurrentLayoutIdx];
	      var scaleroptions = {
	        "originalWidth": v_Layout.width,
	        "originalHeight": v_Layout.height,
	        "previewWidth": document.body.getBoundingClientRect().width,
	        "previewHeight": document.body.getBoundingClientRect().height,
	        "scaleOverride": 0,
	        "widgetDesignWidth": v_Layout.width,
	        "widgetDesignHeight": v_Layout.height,
	        "widgetDesignPadding": 0,
	        "itemsPerPage": 0
	      };
	      
	      utils.scaler("#viewContainer",scaleroptions);
	      
	      // Overlay content
	      //utils.scaler("#overlay-"+v_Layout.id+"-innercontainer-"+v_activeOverlay,scaleroptions);
	    };
	}();

	window.LayoutParser = new function _LayoutParser(){
		/* Splash Layout - This will be displayed if no array v_NewLayouts is empty */
		var SplashLayout = {
				id: "splash",
				width : "1920",
				height : "1080",
				resolutionid : "1",
				bgcolor : "#000000",
				schemaVersion : "1",
				background : "",

				regions : [
				           {
				        	   id : "default_1",
				        	   userId : "1",
				        	   width : "1920",
				        	   height : "1080",
				        	   position : "absolute",
				        	   top : "0",
				        	   left : "0",
				        	   loop : 0,

				        	   media : [
				        	            {
				        	            	id : "0",
				        	            	type : "image",
				        	            	render : "native",
				        	            	duration : "10",
				        	            	lkid : "2226", 
				        	            	userId : "1",
				        	            	schemaVersion : "1",
				        	            	isbase64 : 1,
				        	            	media : {
				        	            		options : [ {
				        	            			uri : [ { jValue : null } ],
				        	            			transIn : [ { jValue : "fadeIn" } ],
				        	            			transInDirection : [ { jValue : "North" } ],
				        	            			transInDuration : [ { jValue : "1500" } ],
				        	            			transOut : [ { jValue : "fadeOut" } ],
				        	            			transOutDirection : [ { jValue : "North" } ],
				        	            			transOutDuration : [ { jValue : "1500" } ]
				        	            		} ]
				        	            	},
				        	            	raw : [],
				        	            	mediaExpired : false
				        	            }
				        	            ]
				           }
				           ]
		};

		/* */



		/* Public functions */
		/**
		 * Get the default layout until Scheduler provide other layouts
		 * @returns {LayoutParser.DefaultLayout} The default layout
		 */

		/**
		 *  @param LayoutID - The id of the layout to be parsed
		 *  @param callback - Callback function
		 *  */
		var loadLayout = function (LayoutID, callback) {

			var xhttp = new XMLHttpRequest();
			xhttp.onreadystatechange = function() {
				if (this.readyState === 4){
					switch (this.status){
					case 0:
					case 200:
						if (this.responseText.length === 0){
							console.error("Empty response on load layout!");
							Logger.cmsLog({category:"error", message: "Could not load layout " + LayoutID.layoutId+".xlf.", onlyOnce:true});
							return;
						}
						var x2jObj = X2J.parseXml(this.responseText);  
						//var parser = new DOMParser();

						//x2jObj.rawjqueryparse = jQueryWebOs.parseXML(this.responseText);
						//x2jObj.rawjqueryparse = parser.parseFromString(this.responseText, "text/xml");
						callback.apply(x2jObj);
						break;
					default:
						Logger.cmsLog({
		                      category:"error", 
		                      message: "Failed to parse layout with Id "+ LayoutID.layoutId + ". Error:"+ this.status + " - " + this.statusText,
		                      method: "loadLayout",
		                      thread: "LayoutParser"
		                  });
						//console.warn("Error "+ this.status + " - " + LayoutID.layoutId+".xlf : " + this.statusText);
					break;
					}
				}
			};

			xhttp.open("GET", "file:///opt/usr/home/owner/content/Downloads/"+LayoutID.layoutId+".xlf", true);
			try {
				xhttp.send();
			} catch (e) {
				throw e;
			}

		};

		/* Parse layout 
		 * @ param LayoutID - The id of the layout to be parsed
		 * */

		var parseLayout = function (LayoutID, callback) {
			loadLayout(LayoutID, function() {
				//console.debug('Layout Log:', this[0].layout[0]);

				/* initiate the regions array to be appended on layout object */
				var regions = [];
	            var isBlacklisted = false;

	            //console.debug('Raw layout parse:',this.rawjqueryparse);
	            /* Get the layout attributes */
	            try {
	              var attributes = this[0].layout[0].jAttr;
	            } catch (e) {
	              //LayoutManager.invalidLayoutIds.push(LayoutID.layoutId); Not applicable in 1.8 version.
				  var objIdx1 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
				  if (objIdx1!==-1){
					LayoutID.layoutId[objIdx1].isValid = false;
					application.statusPage.allLayouts.notifySubscribers();
				  }
	            	isBlacklisted=true;
	            	Logger.cmsLog({
	            		category:"audit", 
	            		message: "Layout with Id "+ LayoutID.layoutId + " has no attributes set. " + e.toString(),
	            		method: "parseLayout",
	            		thread: "LayoutParser",
	            		onlyOnce: true
	            	}); 
	            }
	            
				//var rawjqueryparse = this.rawjqueryparse;
				this[0].layout[0].region.forEach(function(region) {
					/* initiate the media array to be appended on region */
					var regionmedia = [];
					//var rawregion = rawjqueryparse.children[0].children[idx];
	                try {
						if (defined(region.media)) {
							region.media.forEach(function(media) {
								//#23 - Widget Expiry
			                    var shouldAdd = true;
			                    if (media.jAttr.hasOwnProperty('fromDt') && (media.jAttr.hasOwnProperty('toDt'))){
			                      var now = utils.getTimeStamp();
			                      var df = utils.parseDate(media.jAttr.fromDt);
			                      var dt = utils.parseDate(media.jAttr.toDt);
	
			                      if (df>now || now>dt) {
			                        shouldAdd = false;
			                        Logger.cmsLog({
			                            category:"audit", 
			                            message: "Media with id "+media.jAttr.id+" on Region with id "+region.jAttr.id+" and on Layout with Id "+ LayoutID.layoutId + " is expired.",
			                            method: "parseLayout",
			                            thread: "LayoutParser",
			                            onlyOnce: true
			                        });
			                      }
			                    }
		                        if (LibraryManager.blacklisted_widgets.indexOf(media.jAttr.fileId)>-1){
		                            console.warn("Layout with ID "+LayoutID.layoutId+" has blacklisted media with ID " + media.jAttr.fileId);
		                            isBlacklisted = true;
			                        shouldAdd = false;

		                        }
		                        if (shouldAdd){
									var thismedia = {
											id: media.jAttr.id,
											type: media.jAttr.type,
											render: media.jAttr.render,
											duration: media.jAttr.duration,
											media: media,
											enableStat : (media.jAttr.hasOwnProperty('enableStat')?media.jAttr.enableStat:"1")
									};
									regionmedia.push(thismedia);
		                        }
							});
						}
	                } catch (e) {
	                    //LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
	                    //application.statusPage.invalidLayouts_schedule.push(LayoutID.layoutId); //Not applicable in 1.8 version.
						var objIdx2 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
						if (objIdx2!==-1){
						  LayoutID.layoutId[objIdx2].isValid = false;
						  application.statusPage.allLayouts.notifySubscribers();
						}
	                    isBlacklisted=true;
	                    Logger.cmsLog({
	                        category:"audit", 
	                        message: "Region with id "+region.jAttr.id+" and on Layout with Id "+ LayoutID.layoutId + " is erroneous. " + e.toString(),
	                        method: "parseLayout",
	                        thread: "LayoutParser",
	                        onlyOnce: true
	                    });
	                }
					var regionloop;
	                try {
	                    if (defined(region.options)){
							if (region.options[0].hasOwnProperty('loop')) {
								if (region.options[0].loop[0].jValue === "1") {
									regionloop = 1;
								} else {
									regionloop = 0;
								}
							} else {
								regionloop = 0;
							}
	                    }else{
	                        regionloop = 0;
	                      }
                    } catch (e) {
                      //LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
                      //application.statusPage.invalidLayouts_schedule.push(LayoutID.layoutId);
					  var objIdx3 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
					  if (objIdx3!==-1){
						LayoutID.layoutId[objIdx3].isValid = false;
						application.statusPage.allLayouts.notifySubscribers();
					  }

                      isBlacklisted=true;
                      Logger.cmsLog({
                          category:"audit", 
                          message: "Region with id "+region.jAttr.id+" and on Layout with Id "+ LayoutID.layoutId + ". Unable to handle region options. " + e.toString(),
                          method: "parseLayout",
                          thread: "LayoutParser",
                          onlyOnce: true
                      });
                    }

                    try {
						var regionoutput = {
								id : region.jAttr.id,
								userId: "1",
								width: region.jAttr.width,
								height: region.jAttr.height,
								top : region.jAttr.top,
								left : region.jAttr.left,
								loop : regionloop,
								media : regionmedia
						};
                    } catch (e) {
                        //LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
                        //application.statusPage.invalidLayouts_schedule.push(LayoutID.layoutId);
						var objIdx4 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
						if (objIdx4!==-1){
						  LayoutID.layoutId[objIdx4].isValid = false;
						  application.statusPage.allLayouts.notifySubscribers();
						}

                        isBlacklisted=true;
                        Logger.cmsLog({
                            category:"audit", 
                            message: "Unable to set region attributes on Region with id "+region.jAttr.id+" and on Layout with Id "+ LayoutID.layoutId + ". " + e.toString(),
                            method: "parseLayout",
                            thread: "LayoutParser",
                            onlyOnce: true
                        });
                    }
					if (defined(region.jAttr.zindex)){
						regionoutput.zindex = region.jAttr.zindex;
					}
					regions.push(regionoutput);
				});

				/* Preload the background of the layout to avoid black screen between layout switching */

				/* This actually does nothing! */
//				if(defined(attributes.background)) {
//				jQueryWebOs.preload("http://127.0.0.1:9080/"+attributes.background);
//				}
	            try {
					var layout = {
							id : LayoutID.layoutId,
							scheduleid : LayoutID.scheduleId,
							width : attributes.width,
							height : attributes.height,
							resolutionid : "1",
							bgcolor : attributes.bgcolor,
							schemaVersion : attributes.schemaVersion,
							regions : regions,
							enableStat : (attributes.hasOwnProperty('enableStat')?attributes.enableStat:"1")
					};
					if (defined(attributes.background)) {
						layout.background = "file:///opt/usr/home/owner/content/Downloads/"+attributes.background;
					} else {
						layout.background = "";
					}
	            } catch (e) {
	            	//LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
	                //application.statusPage.invalidLayouts_schedule.push(LayoutID.layoutId);
					var objIdx5 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
					if (objIdx5!==-1){
					  LayoutID.layoutId[objIdx5].isValid = false;
					  application.statusPage.allLayouts.notifySubscribers();
					}
                	isBlacklisted=true;
                	Logger.cmsLog({
                		category:"audit", 
                		message: "Unable to set layout attributes on Layout with Id "+ LayoutID.layoutId + ". " + e.toString(),
                		method: "parseLayout",
                		thread: "LayoutParser",
                		onlyOnce: true
                	});
	            } 
	            
				if (!isBlacklisted){
	                callback.apply(layout);
	            }else{
	                callback.apply(null);
	            }
			});
		};

		this.getDefault = function(){
			var splashimage = LayoutParser.SplashImage();
			SplashLayout.regions[0].media[0].media.options[0].uri[0].jValue = splashimage;

//			var o = DeviceManager.Device.getOrientation();
//			if (o === DeviceManager.Device.SCREENORIENTATION_PORTRAIT || o === DeviceManager.Device.SCREENORIENTATION_REVERSE_PORTRAIT){
			SplashLayout.width = screen.width;
			SplashLayout.height = screen.height;
			SplashLayout.regions[0].width = screen.width;
			SplashLayout.regions[0].height = screen.height;
//			}
			return SplashLayout;
		};

		this.parse = function(cbSuccess, cbFailure, options){
			var layouts = [];
			var layoutsindex = -1;
			function nextlayout() {
				layoutsindex++;
				if (typeof options.layouts[layoutsindex] !== 'undefined') {
					parseLayout(options.layouts[layoutsindex], function () {
	                    if (this !== null){
	                        var shouldAdd = true;
	                        for (var i = 0; i < this.regions.length; i++) {
	                          if (this.regions[i].media.length === 0){
	                            shouldAdd = false;
	                             Logger.cmsLog({
	                                  category:"error", 
	                                  message: "Region "+this.regions[i].id+" is empty on Layout with Id "+ options.layouts[layoutsindex].layoutId + ".",
	                                  method: "parse",
	                                  thread: "LayoutParser",
	                                  onlyOnce:true
	                              });
	                            break;
	                          } 
	                        }
	                        var invalidLayoutIdIndex = LayoutManager.invalidLayoutIds.indexOf(options.layouts[layoutsindex].layoutId);
	                        if (shouldAdd){
	                        	layouts.push(this);
	                            if (invalidLayoutIdIndex>-1){
	                                LayoutManager.invalidLayoutIds.splice(invalidLayoutIdIndex,1);
	                            }
                            }else{
	                              if (invalidLayoutIdIndex===-1){
	                                LayoutManager.invalidLayoutIds.push(options.layouts[layoutsindex].layoutId);
	                              }
	                        }
	                    }
						nextlayout();
					});
				} else {
					cbSuccess.apply(layouts);
				}
			}
			nextlayout();
		};
	}();
	//jshint +W057
}());
