/* global LayoutManager */

/**
 * Load and play an overlay audio of an element.<br>
 * @param {Object} media The audio information.
 * @param {Object} container Not Used.
 * @returns {LayoutManager.audio} Return an audio object. 
 */
 (function() {
	"use strict";

	window.LayoutManager.audio = function (media, container, callback, layoutid, scheduleid, scp) {
		var parent = this;
		this.media = media;
		this.container = container;
		var source = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;
		var loop = 0;
		try {
			loop = parseInt(media.media.options[0].loop[0].jValue);	
		} catch (error) {
			console.warn(error);
			loop = 0;
		}
		this.duration = parseInt(media.duration);
		/* Create a buffer source for the specific file. */
		var bufSource = LayoutManager.audioContext.createBufferSource(); // creates a sound source

		/* Create a gain node */
		var gain = LayoutManager.audioContext.createGain();

		/* Wire nodes */
		bufSource.connect(gain); // connect the source to the gain node for volume setting.
		gain.connect(LayoutManager.audioContext.destination); // connect the gain node to the context's destination (the speakers)

		this.preload = function(resolveCb, failureCb){
			//Logger.log("Preload " + source, Logger.DEBUG);

			var request = new XMLHttpRequest();
			request.open('GET', source, true);

			request.responseType = 'arraybuffer';

			// Decode asynchronously
			request.onload = function() {
				//Logger.log("Loaded " + source, Logger.DEBUG);
				LayoutManager.audioContext.decodeAudioData(request.response,
						function(buffer) {
					//Logger.log("Decoded " + source, Logger.DEBUG);
					bufSource.buffer = buffer;
					if(parent.duration === 0){
						getDuration(source, function(duration){
							parent.duration = duration;
							if (resolveCb){
								resolveCb();
							}
						});
					}else{
						if (resolveCb){
							resolveCb();
						}
					}
						
				},
				function(evt){
					//Logger.log("error", Logger.WARN, evt);
					if (failureCb){
						failureCb(evt);
					}
				});
			};

			request.send();
		};

		this.timeoutMedia = function () {
			parent.timeoutElement = setTimeout(function(){
				if (scheduleid && media.enableStat !== "0"){
					Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
				}
				parent.stop();
				callback();
			}, parent.duration * 1000, layoutid, scp);
		};
		var timeoutMediaInvoke = this.timeoutMedia;

		function getDuration(src, cb) {
			var audio = new Audio(src);
			var internalLoadedMetadata = function(){
				audio.removeEventListener("loadedmetadata", internalLoadedMetadata);
				var d = audio.duration;
				audio = null;
				cb(d);
			}
			audio.addEventListener("loadedmetadata", internalLoadedMetadata);
		};

		this.play = function(){
			//Logger.log("Play " + source, Logger.DEBUG);
			bufSource.onended = function(){
				//Logger.log("Ended: "+ source, Logger.DEBUG);
				bufSource.stop();
				bufSource = null;
			};
			bufSource.start(0);                     
			timeoutMediaInvoke(parent);
		};

		this.stop = function(){
			//Logger.log("Stop " + source,Logger.DEBUG);
			if (bufSource) {//If source buffer is valid.
				bufSource.stop();
			}
		};

		if (media.volume){
			try {
				gain.gain.value = Number(media.volume)/100;
			} catch (e) {
				gain.gain.value = 1;
			}
		};

		if(loop){
			if (Number(loop) === 1){
				bufSource.loop = true;
			}
		};

		this.preload(function(){
			//timeoutMediaInvoke(parent);
			parent.play();
		}, null);
		
		this.stopMediaTimeout = function () {
			if (parent){
				parent.stop();
				clearTimeout(parent.timeoutElement);
			}
		};
	};
}());