/* Tizen - Xibo | Native Video Module */

/* global LayoutManager, Logger, utils, LibraryManager, DeviceManager */

/**
 * Display a html package widget in layout
 * @param {object} media Media options
 * @param {HTMLElement} container The parent (HTML) element.
 * @param {funtion} callback Callback function to call on end.
 * @param {string} layoutid The layout id
 * @param {string} scheduleid The schedule id
 * @param {string} scp "Main" if is is a normal layout or "overlay" if it is an overlay layout.
 * @returns {LayoutManager.htmlpackage} An html package widget.
 */
LayoutManager.htmlpackage = function (media, container, callback, layoutid, scheduleid, scp) {
  try {
    
    if (media.media.audio !== undefined) {
        var audio = {
            uri : "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
            loop : media.media.audio[0].uri[0].jAttr.loop,
            volume : media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function(){
            parent.audiooverlay.play();
        }, null);
    }
    
    this.isready = true;
    this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;
    this.uri = this.uri.substr(0, this.uri.length-4);
    
    if (media.media.options[0].hasOwnProperty('nominatedFile')){
      this.uri += "/" + decodeURIComponent(media.media.options[0].nominatedFile[0].jValue);
    }else{
      this.uri += "/index.html";
    }
    
    this.statsStartTime = utils.getISODate();
    this.duration = parseInt(media.duration);
    var duration = this.duration;
    this.id = media.media.options[0].uri[0].jValue.substr(0,media.media.options[0].uri[0].jValue-4);
    this.elem = document.createElement("iframe");
    this.elem.src = this.uri;
    this.elem.style.position = "absolute";
    this.elem.style.left = 0;
    this.elem.style.top = 0;
    this.elem.style.width = "100%";
    this.elem.style.height = "100%";
    this.elem.style.overflow = "hidden";
    this.elem.style.border = "none";
    var parent = this;
    this.timeoutElement = null;
    this.removeMedia = function () { 
      container.removeChild(this.elem); 
    };
    var hastransIn = false;
    var hastransOut = false;
    var transInDuration = 1000;
    var transOutDuration = 1000;
    var transInType = null;
    var transOutType = null;
    var transInDirection = "S";
    var transOutDirection = "S";
    
		if (media.media.options[0].hasOwnProperty("transIn")){
          hastransIn = true; 
          transInType = media.media.options[0].transIn[0].jValue;
          if(media.media.options[0].transInDirection){
              transInDirection = media.media.options[0].transInDirection[0].jValue;
          }
          if (media.media.options[0].hasOwnProperty("transInDuration")){ 
            transInDuration = media.media.options[0].transInDuration[0].jValue; 
          }
        }
		if (media.media.options[0].hasOwnProperty("transOut")){ 
          hastransOut = true; 
          transOutType = media.media.options[0].transOut[0].jValue;
          if (media.media.options[0].transOutDirection){
            transOutDirection = media.media.options[0].transOutDirection[0].jValue; 
          }
          if (media.media.options[0].hasOwnProperty("transOutDuration")){ 
            transOutDuration = media.media.options[0].transOutDuration[0].jValue; 
          }
        }
        this.timeoutMedia = function () {
        //var tag = "r: " + container.getAttribute("id") + " m: " + media.id + " html";
        this.timeoutElement = setTimeout(function(){
            if (hastransOut) {
                LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function() { 
                    if (parent.audiooverlay){
                        parent.audiooverlay.stop();
                    }
                    if (scheduleid && media.enableStat !== "0"){
                        Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
                    }
                    callback(); 
                });
            } else {
                if (parent.audiooverlay){
                    parent.audiooverlay.stop();
                }
                if (scheduleid && media.enableStat !== "0"){
                    Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
                }
                callback();
            }
        }, duration * 1000, layoutid, scp);
    };
    var timeoutMediaInvoke = this.timeoutMedia;
    /**
     * Display the media on the layout.
     * @returns {undefined}
     */
    this.displayThisMedia = function () {
        if (hastransIn) { duration -= parseInt(transInDuration)/1000; utils.hide(this.elem);}
        if (hastransOut) { duration -= parseInt(transOutDuration)/1000; }
        if (duration < 0) { duration = 0; }
        container.innerHTML = null;
        container.appendChild(this.elem);
        if (hastransIn) {
            LayoutManager.transin(this.elem, container, transInType, transInDuration, transInDirection, function() {timeoutMediaInvoke(parent);});
        } else {
            timeoutMediaInvoke(parent);
        }
    };
    this.stopMediaTimeout = function () {
        if (parent.audiooverlay){
            parent.audiooverlay.stop();
        }
//        if (scheduleid){
//            Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
//        }
        clearTimeout(parent.timeoutElement);
    };
    
    this.displayThisMedia();
  } catch (e) {
    Logger.cmsLog({
        category:"error", 
        message: "Problem displaying html package with id "+media.media.options[0].uri[0].jValue+" on layout with Id "+ layoutid + ". Error:"+ e,
        method: "factory",
        thread: "LayoutManager.htmlpackage",
        onlyOnce: true
    });
    callback();
  }
};
