/* Tizen - Xibo | Native Video Module */

/* global LayoutManager, LibraryManager, utils, Logger */

(function() {
	"use strict";

	LayoutManager.video = function (media, container, callback, layoutid, scheduleid, scp) {
		if (media.media.audio !== undefined) {
			var audio = {
					uri : "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
					loop : media.media.audio[0].uri[0].jAttr.loop,
					volume : media.media.audio[0].uri[0].jAttr.volume
			};
			this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
			this.audiooverlay.preload(function(){
				parent.audiooverlay.play();
			}, null);
		}

		this.statsStartTime = utils.getISODate();
		this.isready = true;
		this.usePreload = false;
		this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;
		
	    var isFullscreen = false;
	    var previousRegionSize = {};
	    //XLG-118: Video Full Screen Option
	    if (media.media.options[0].hasOwnProperty("showFullScreen")){
	      if(media.media.options[0].showFullScreen[0].jValue[0] === "1"){
	        isFullscreen = true;
	        previousRegionSize = {
	          width: container.style.width,
	          height: container.style.height,
	          top: container.style.top,
	          left: container.style.left,
	          zIndex: container.style.zIndex
	        };
	      }
	    }
	    var restoreSize = function(){
	        container.style.width = previousRegionSize.width;
	        container.style.height = previousRegionSize.height;
	        container.style.top = previousRegionSize.top;
	        container.style.left = previousRegionSize.left;
	        container.style.zIndex = previousRegionSize.zIndex;
	    };
	    
		
		this.elem = document.createElement("video");
		this.elem.setAttribute('autoplay','');
		this.elem.setAttribute('texture','');
		this.elem.setAttribute('preload','metadata');
		this.duration = parseInt(media.duration) * 1000;
		var parent = this;
		var duration = this.duration;
		if (media.media.options[0].loop !== undefined) { if (media.media.options[0].loop[0].jValue === '1') { this.elem.setAttribute('loop',''); } }
		if (media.media.options[0].mute !== undefined) { if (media.media.options[0].mute[0].jValue === '1') { this.elem.setAttribute('muted',''); } }
		this.elem.style.left = "0";
		this.elem.style.top = "0";
		this.elem.style.width = "100%";
		this.elem.style.height = "100%";
		if (media.media.options[0].scaleType !== undefined) {
			if (media.media.options[0].scaleType[0].jValue === "aspect") {
				this.elem.style.objectFit = "contain";
			} else if (media.media.options[0].scaleType[0].jValue === "stretch") {
				this.elem.style.objectFit = "fill";
			} else {
				this.elem.style.objectFit = "contain";
			}
		} else {
			this.elem.style.objectFit = "contain";
		}
		var videoelement = document.createElement("source"); 
		videoelement.src = this.uri;
		//videoelement.type = mime;
		this.elem.appendChild(videoelement);
		this.elem.onerror = function () { 
			parent.isready = true; 
	        if (isFullscreen){
	            restoreSize();
	        }
	        LibraryManager.blacklisted_widgets.push(media.media.jAttr.fileId);
	        Logger.cmsLog({
	            category:"audit", 
	            message:"Could not play video: Widget ID "+media.id+" with name "+media.media.options[0].uri[0].jValue+" on Layout ID "+layoutid+", in Region ID "+container.id+ ". ",
	            layoutID: layoutid,
	            mediaID: media.id,
	            method:"onerror",
	            thread:"LayoutManager.video",
	            onlyOnce:true
	        });
	        callback();
        };
		this.elem.onload = function () { parent.isready = true; };
		this.timeoutElement = null;
		this.timeoutMedia = function () {
			//var tag = "r: " + container.getAttribute("id") + " m: " + media.id + " image";
			parent.timeoutElement = setTimeout(function(){
				//console.info("VDO: Video timeoutMedia fired!");
				if (hastransOut) {
					LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function() { 
						if (parent.audiooverlay){
							parent.audiooverlay.stop();
						}
						//container.innerHTML = null;
						//parent = null;
						if (scheduleid && media.enableStat !== "0"){
	                        Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
	                    }
	                    if (isFullscreen){
	                        restoreSize();
	                    }
						callback(); 
					});
				} else {
					if (parent.audiooverlay){
						parent.audiooverlay.stop();
					}
					//container.innerHTML = null;
					//parent = null;
					if (scheduleid && media.enableStat !== "0"){
                        Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
                    }
					if (isFullscreen){
                        restoreSize();
                    }
					callback();
				}
			}, duration, layoutid, scp);
		};
		var timeoutMediaInvoke = this.timeoutMedia;
		this.removeMedia = function () { 
			//console.info("VDO: Remove video element!");
			container.removeChild(this.elem); 
		};
		var hastransIn = false;
		var hastransOut = false;
		var transInDuration = 1000;
		var transOutDuration = 1000;
		var transInType = null;
		var transOutType = null;
		var transInDirection = "S";
		var transOutDirection = "S";
		if (media.media.options[0].hasOwnProperty("transIn")){
          hastransIn = true; 
          transInType = media.media.options[0].transIn[0].jValue;
          if(media.media.options[0].transInDirection){
              transInDirection = media.media.options[0].transInDirection[0].jValue;
          }
          if (media.media.options[0].hasOwnProperty("transInDuration")){ 
            transInDuration = media.media.options[0].transInDuration[0].jValue; 
          }
        }
		if (media.media.options[0].hasOwnProperty("transOut")){ 
          hastransOut = true; 
          transOutType = media.media.options[0].transOut[0].jValue;
          if (media.media.options[0].transOutDirection){
            transOutDirection = media.media.options[0].transOutDirection[0].jValue; 
          }
          if (media.media.options[0].hasOwnProperty("transOutDuration")){ 
            transOutDuration = media.media.options[0].transOutDuration[0].jValue; 
          }
        }
		this.displayThisMedia = function () {
			//console.info("VDO: Display video element!");
			if (parent.usePreload){
				container.removeChild(container.lastChild);
			}
			if (isFullscreen){
		        container.style.width = "100%";
		        container.style.height = "100%";
		        container.style.top = "0";
		        container.style.left = "0";
		        container.style.zIndex = 99999999;
			}
	        timeoutMediaInvoke(parent);
		};
		this.elem.onloadeddata = function() {
			//console.info("VDO: Video Loaded! Duration: " + parent.elem.duration);
			if (duration === 0) {
				duration = parent.elem.duration * 1000;
			}
			if (hastransIn) { duration -= parseInt(transInDuration);}
			if (hastransOut) { duration -= parseInt(transOutDuration); }     
			if (duration < 0) { duration = 0; }

			//console.debug("Duration: " + duration);
			if (hastransIn) {
				LayoutManager.transin(parent.elem, container, transInType, transInDuration, transInDirection, function() {
					parent.displayThisMedia();
					//timeoutMediaInvoke(parent);
				});
			} else {
				parent.displayThisMedia(parent);
			}

		};
		this.stopMediaTimeout = function () {
			//console.info("VDO: Clear video element timeout!", parent.timeoutElement);
			if (parent){
				if (parent.audiooverlay){
					parent.audiooverlay.stop();
				}
//				if (scheduleid){
//                    Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
//                }
				clearTimeout(parent.timeoutElement);
				console.debug('Stop Media Timeout for element ',parent.timeoutElement);
			}
            if (isFullscreen){
                restoreSize();
            }

			//parent = null;
		};

		if (hastransIn){ utils.hide(this.elem);}

		// Issue #36: Consecutive videos are not shown due to firmware problem.
		if (!container){
			return;
		}
		if (container.children.length>0){
			if (container.lastChild.tagName === 'VIDEO'){
				//console.info("VDO: Previous media IS video. Don't use preload!");
				container.innerHTML = null;
				container.appendChild(this.elem);
			}else{
				//console.info("VDO: Previous media not video. Use preload!");
				this.usePreload = true;
				container.insertBefore(this.elem,container.firstElementChild);
			}
		}else{
			//console.info("VDO: No previous media. Append!");
			container.appendChild(this.elem);
		}
	};
}());
